package org.jruby.webapp;
import org.jruby.main.ASTSerializer;
import java.io.File;
import java.io.IOException;
/**
 * Allows AST serialization to be done from the command-line.
 * This should be added to ASTSerializer.
 *
 * @author Robert Egglestone
 */
public class ASTSerializerMain {

	private static final String DEFAULT_SUFFIX = ".ast.ser";
	private static boolean replace = false;

	public static void main(String[] args) {
		// usage
		int argIndex = 0;
		if (args.length == 0) usage();
		// replace?
		if (args[argIndex].equals("--replace")) {
			argIndex++;
			replace = true;
		}
		// check the source
		if (args.length == argIndex) usage();
		File source = new File(args[argIndex++]);
		if (!source.exists()) {
			System.err.println("The source file could not be found");
			System.exit(1);
		}
		// check the target
		File target = null;
		if (args.length != argIndex) {
			target = new File(args[argIndex++]);
			if (source.isDirectory() != target.isDirectory()) {
				System.err.println("If source is a file, target must also be a file");
				System.err.println("If source is a directory, target must also be a directory");
				System.exit(1);
			}
		}
		// preparse the file
		if (source.isFile()) {
			preparse(source, target);
		} else if (source.isDirectory()) {
			preparseAll(source, target);
		}
		// done
		System.exit(0);
	}

	private static void preparseAll(File source, File target) {
		if (target == null) target = source;
		File[] children = source.listFiles();
		for(int i=0; i<children.length; i++) {
			File child = children[i];
			if (child.isDirectory()) {
				preparseAll(child, new File(target, child.getName()));
			} else if(child.getName().endsWith(".rb")) {
				preparse(child, new File(target, child.getName() + DEFAULT_SUFFIX));
			}
		}
	}

	private static void preparse(File source, File target) {
		if (target == null) {
			target = new File(source.getPath() + DEFAULT_SUFFIX);
		}
		try {
			ASTSerializer.serialize(source, target);
			if (replace) source.delete();
		} catch (NullPointerException e) {
			System.err.println("Warning: failed to preparse " + source.getPath());
		} catch (IOException e) {
			System.err.println("Warning: failed to preparse " + source.getPath() + ": " + e.getMessage());
		}
	}

	private static void usage() {
		System.out.println("Usage: ASTSerializerMain [--replace] source [target]");
		System.out.println("Preparses the Ruby file source, and stores the result in target");
		System.out.println("Target defaults to source.ast.ser");
		System.exit(1);
	}

}
