package org.jruby.webapp;
import org.jruby.exceptions.RaiseException;
import org.jruby.RubyException;
import org.jruby.NativeException;
import org.jruby.RubyArray;
import org.jruby.RubyString;
import org.jruby.Ruby;
import org.jruby.javasupport.JavaEmbedUtils;
import org.jruby.runtime.builtin.IRubyObject;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.ArrayList;
/**
 * This is an exception from Ruby, tweaked to present useful information to the user.
 * @author Robert Egglestone
 */
public class JRubyException extends RuntimeException {

	public JRubyException(RaiseException e) {
		super(message(e), cause(e));
		createStackTrace(e);
	}

	private void createStackTrace(RaiseException e) {
		// this is horrible, horrible code
		// the ruby exceptions don't directly match to Java stack elements,
		// because the class and method information is not exposed (although it is available)
		RubyException rubyCause = e.getException();
		Ruby runtime = rubyCause.getRuntime();

		RubyArray backtraceArray = (RubyArray)rubyCause.backtrace();
		IRubyObject[] backtrace = backtraceArray.toJavaArray();
		ArrayList stackTrace = new ArrayList(backtrace.length / 2);
		for (int i=0; i<backtrace.length-1; i++) {
			IRubyObject rubyLine = backtrace[i];
			if (rubyLine instanceof RubyString) {
				// break down the ruby stack trace into what it really contains
				String line = (String)JavaEmbedUtils.rubyToJava(runtime, rubyLine, String.class);
				Matcher matcher = Pattern.compile("^(.*):(-?\\d+)$").matcher(line);
				if (!matcher.matches()) continue;

				String file = matcher.group(1);
				int lineNumber = Integer.parseInt(matcher.group(2));
				// and decide what we want to present to java
				String archiveFile = "<builtin>";
				String relativeFile = file;
				int internalPathPos = file.indexOf("!/");
				if (internalPathPos != -1) {
					int previous = file.lastIndexOf("/", internalPathPos);
					if (previous != -1) {
						archiveFile = file.substring(previous + 1, internalPathPos);
					}
					relativeFile = file.substring(internalPathPos + 1);
				}
				stackTrace.add(new StackTraceElement("<ruby>", archiveFile, relativeFile, lineNumber));
			}
		}
		StackTraceElement[] stackTraceArr = (StackTraceElement[])stackTrace.toArray(new StackTraceElement[0]);
		setStackTrace(stackTraceArr);
	}

	private static String message(RaiseException e) {
		String message = e.getMessage();
		if (message == null) {
			RubyException rubyCause = e.getException();
			IRubyObject rubyMessage = rubyCause.message;
			Ruby runtime = rubyMessage.getRuntime();
			message = (String)JavaEmbedUtils.rubyToJava(runtime, rubyMessage, String.class);
		}
		return message;
	}

	private static Throwable cause(RaiseException e) {
		RubyException rubyCause = e.getException();
		if (rubyCause instanceof NativeException) {
			NativeException ne = (NativeException)rubyCause;
			Ruby runtime = ne.getRuntime();
			return (Throwable)JavaEmbedUtils.rubyToJava(runtime, ne.cause(null), Throwable.class);
		}
		return null;
	}

}
